import unittest
import pandas as pd
import numpy as np
from SimplyEDA import remove_outlier, find_specialchar, vif_cal, dups, boxplt_continous, enhance_summary

class TestSimplyEDA(unittest.TestCase):

    def setUp(self):
        # Setup a sample DataFrame for testing
        self.df = pd.DataFrame({
            'A': [1, 2, 2, 4, 5, 6, 7, 8, 9, 10],
            'B': [11, 12, 13, 14, 15, 16, 17, 18, 19, 20],
            'C': [21, 22, 23, 24, 25, 26, 27, 28, 29, 30],
            'D': ['a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j']
        })

    def test_remove_outlier(self):
        lower, upper = remove_outlier(self.df['A'])
        self.assertEqual(lower, -4.0)
        self.assertEqual(upper, 16.0)

    def test_find_specialchar(self):
        # Capture the print output
        import io
        import sys
        captured_output = io.StringIO()
        sys.stdout = captured_output
        find_specialchar(self.df)
        sys.stdout = sys.__stdout__
        output = captured_output.getvalue()
        self.assertIn('The unique values in A are as below', output)

    def test_vif_cal(self):
        from statsmodels.stats.outliers_influence import variance_inflation_factor
        # Check if function runs without error
        try:
            vif = vif_cal(self.df[['A', 'B', 'C']])
            self.assertIsInstance(vif, pd.DataFrame)
        except Exception as e:
            self.fail(f"vif_cal raised Exception unexpectedly: {e}")

    def test_dups(self):
        # Create a DataFrame with duplicates
        df_with_dups = self.df.append(self.df.iloc[0])
        # Capture the print output
        import io
        import sys
        captured_output = io.StringIO()
        sys.stdout = captured_output
        dups(df_with_dups)
        sys.stdout = sys.__stdout__
        output = captured_output.getvalue()
        self.assertIn('Number of duplicate rows = 1', output)

    def test_boxplt_continous(self):
        # Check if function runs without error
        try:
            boxplt_continous(self.df[['A', 'B', 'C']])
        except Exception as e:
            self.fail(f"boxplt_continous raised Exception unexpectedly: {e}")

    def test_enhance_summary(self):
        summary = enhance_summary(self.df, custom_percentiles=[5, 95])
        self.assertIsInstance(summary, pd.DataFrame)
        self.assertIn('IQR', summary.columns)
        self.assertIn('LW', summary.columns)
        self.assertIn('UW', summary.columns)
        self.assertIn('Outliers', summary.columns)
        self.assertIn('Duplicates', summary.columns)
        self.assertIn('Missing', summary.columns)
        self.assertIn('Skew', summary.columns)
        self.assertIn('Skew_Category', summary.columns)
        self.assertIn('5%', summary.columns)
        self.assertIn('95%', summary.columns)

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# coding: utf-8

import seaborn as sns
import matplotlib.pyplot as plt
import numpy as np
import pandas as pd

def remove_outlier(series):
    Q1 = series.quantile(0.25)
    Q3 = series.quantile(0.75)
    IQR = Q3 - Q1
    lower_bound = Q1 - 1.5 * IQR
    upper_bound = Q3 + 1.5 * IQR
    return lower_bound, upper_bound

def find_specialchar(dataframe):
    for column in dataframe.columns:
        print(f"The unique values in {column} are as below\n", dataframe[column].unique())

def vif_cal(dataframe):
    from statsmodels.stats.outliers_influence import variance_inflation_factor
    variables = dataframe.columns
    vif = pd.DataFrame()
    vif["VIF"] = [variance_inflation_factor(dataframe[variables].values, i) for i in range(dataframe[variables].shape[1])]
    vif["Features"] = variables
    return vif

def dups(dataframe):
    print(f'Number of duplicate rows = {dataframe.duplicated().sum()}')

def boxplt_continous(dataframe):
    import matplotlib.pyplot as plt
    import seaborn as sns
    sns.set(style="whitegrid")
    for column in dataframe.select_dtypes(include=np.number).columns:
        plt.figure(figsize=(10, 5))
        sns.boxplot(x=dataframe[column])
        plt.title(f'Boxplot of {column}')
        plt.show()

def enhance_summary(dataframe, custom_percentiles=[]):
    # Summary for numerical columns
    numeric_summary = dataframe.describe(percentiles=[p/100 for p in custom_percentiles]).T
    numeric_summary['IQR'] = numeric_summary['75%'] - numeric_summary['25%']
    numeric_summary['LW'] = numeric_summary['25%'] - 1.5 * numeric_summary['IQR']
    numeric_summary['UW'] = numeric_summary['75%'] + 1.5 * numeric_summary['IQR']

    for column in dataframe.select_dtypes(include=np.number).columns:
        lower_range = numeric_summary.loc[column, 'LW']
        upper_range = numeric_summary.loc[column, 'UW']
        outliers = (dataframe[column] < lower_range) | (dataframe[column] > upper_range)
        num_outliers = np.sum(outliers)
        numeric_summary.loc[column, 'Outliers'] = num_outliers

    # Summary for categorical columns
    categorical_summary = dataframe.describe(include=['O']).T
    categorical_summary['Unique'] = dataframe.select_dtypes(include=['O']).nunique()
    categorical_summary['Top'] = dataframe.select_dtypes(include=['O']).mode().iloc[0]
    categorical_summary['Freq'] = dataframe.apply(lambda x: x.value_counts().iloc[0] if x.dtype == 'O' else np.nan)

    # Combine numerical and categorical summaries
    summary = pd.concat([numeric_summary, categorical_summary], axis=0, sort=False)
    
    # Add the number of duplicates and missing values
    for column in dataframe.columns:
        summary.loc[column, 'Duplicates'] = dataframe[column].duplicated().sum()
        summary.loc[column, 'Missing'] = dataframe[column].isnull().sum()

    # Add skewness column and skewness category column for numerical columns
    for column in dataframe.select_dtypes(include=np.number).columns:
        skew_value = dataframe[column].skew()
        summary.loc[column, 'Skew'] = round(skew_value, 2)

        if skew_value >= 1:
            summary.loc[column, 'Skew_Category'] = 'Positive'
        elif skew_value <= -1:
            summary.loc[column, 'Skew_Category'] = 'Negative'
        elif -0.5 <= skew_value <= 0.5:
            summary.loc[column, 'Skew_Category'] = 'Normal'
        else:
            summary.loc[column, 'Skew_Category'] = 'Undefined'

    return summary

# Example usage
if __name__ == "__main__":
    # Sample DataFrame
    data = {'A': [1, 2, 2, 4, 5, 6, 7, 8, 9, 10],
            'B': [11, 12, 13, 14, 15, 16, 17, 18, 19, 20],
            'C': [21, 22, 23, 24, 25, 26, 27, 28, 29, 30],
            'D': ['a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j']}
    df = pd.DataFrame(data)

    # Enhanced summary
    summary = enhance_summary(df, custom_percentiles=[5, 95])
    print(summary)



from setuptools import setup
import pathlib

# The directory containing this file
HERE = pathlib.Path(__file__).parent

# The text of the README file
README = (HERE / "README.md").read_text()

setup(
    name='SimplyEDA',
    version='0.1.5',  # Update the version number
    description='A simple library for exploratory data analysis',
    long_description=README,
    long_description_content_type='text/markdown',
    url='https://github.com/kitranet/SimpleEDA',
    author='M.R.Vijay Krishnan',
    author_email='vijaykrishnanmr@gmail.com',
    py_modules=['SimplyEDA'],
    install_requires=[
        'seaborn',
        'matplotlib',
        'numpy',
        'pandas',
        'statsmodels'
    ],
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
    keywords='EDA data-analysis',
    python_requires='>=3.6',
)